#
# EDOS testing framework
# Copyright (C) 2006 EDGE-IT
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

#
# Author: Francois Dechelle <fdechelle@mandriva.com>
#

import locale
import qt
import sys
import datetime
from edostestassistant.test import Test
from edostestassistant.LangPopUp import LangPopUp
from edostestassistant.Start import Start
from edostestassistant.Step import Step
from edostestassistant.Result import Result
from edostestassistant.defaults import defaults
from edostestrunner.testreporter import makeTestReportId

class NextWidgetAdapter:
    def __init__(self,qApplication,widget,button,nextWidget):
        self.qApplication = qApplication
        widget.connect( button, qt.SIGNAL("clicked()"), self.move)
        self.nextWidget = nextWidget
        
    def move(self):
        w = self.qApplication.mainWidget()
        p = w.pos()
        s = w.size()
        # FIXME: sometimes, it does not work
        self.nextWidget.resize(s)
        self.nextWidget.move(p)
        self.nextWidget.show()
        self.qApplication.setMainWidget( self.nextWidget)
        w.hide()


class StartAdapter:
    def __init__(self, testAssistant):
        self.testAssistant = testAssistant
        w = self.testAssistant.startWidget
        w.testSuiteLabel.setText( testAssistant.test.suite)
        w.testLabel.setText( testAssistant.test.name)
        try:
            description_text = testAssistant.test.description[ testAssistant.language.getLanguage() ]
        except:
            description_text = testAssistant.test.description['en']
        w.descriptionLabel.setText(description_text)
    

class StepAdapter:
    def __init__(self, testAssistant):
        self.testAssistant = testAssistant
        w = self.testAssistant.stepWidget
        w.connect( w.nextButton, qt.SIGNAL("clicked()"), self.nextStep)
        w.connect( w.previousButton, qt.SIGNAL("clicked()"), self.previousStep)
        self.update( 0)

    def previousStep(self):
        self.update(self.counter - 1)

    def nextStep(self):
        self.update(self.counter + 1)

    def update(self, counter):
        test = self.testAssistant.test
        if counter >= 0 and counter < len(test.steps):
            self.counter = counter
            language = self.testAssistant.language.getLanguage()
            t = str(self.counter + 1) + '/' + str(len(test.steps))
            self.testAssistant.stepWidget.stepNumberLabel.setText( t)
            try:
                steps = test.steps[self.counter][language]
            except KeyError:
                steps = test.steps[self.counter]['en']
            self.testAssistant.stepWidget.instructionsText.setText(steps)
            atBeginning = (counter == 0)
            self.testAssistant.stepWidget.previousButton.setEnabled( not atBeginning)
            atEnd = (counter == len(test.steps) - 1)
            self.testAssistant.stepWidget.nextButton.setEnabled( not atEnd)
            self.testAssistant.stepWidget.resultButton.setEnabled( atEnd)


class ResultAdapter:
    def __init__(self, testAssistant):
        self.testAssistant = testAssistant
        w = self.testAssistant.resultWidget
        w.connect( w.notExecutedRadioButton,qt.SIGNAL("stateChanged(int)"), self.notExecuted)
        w.connect( w.passedRadioButton,qt.SIGNAL("stateChanged(int)"), self.passed)
        w.connect( w.failedRadioButton,qt.SIGNAL("stateChanged(int)"), self.failed)
        w.connect( w.submitButton, qt.SIGNAL("clicked()"), self.submit)
        w.connect( w.commentText, qt.SIGNAL("clicked(int,int)"), self.comment)
        self.result = 'Not executed'
        self.commentChanged = False
        try:
            result_text = testAssistant.test.expectedResult[ testAssistant.language.getLanguage()]
        except KeyError:
            result_text = testAssistant.test.expectedResult['en']
        
        w.resultText.setText(result_text)

    def notExecuted(self,i):
        if i != 0:
            self.result = 'Not executed'

    def passed(self,i):
        if i != 0:
            self.result = 'Passed'

    def failed(self,i):
        if i != 0:
            self.result = 'Failed'

    def comment(self,x,y):
        self.testAssistant.resultWidget.commentText.setText("")
        self.commentChanged = True

    def report(self):
        testReportId = makeTestReportId( self.testAssistant.test.name, self.testAssistant.test.suite)
        self.testAssistant.testReporter.reportRunning(
            test_report_id = testReportId,
            test_suite_name = self.testAssistant.test.suite,
            test_name = self.testAssistant.test.name,
            platform_identification = 'None',
            # FIXME
            tester_username = 'NoName',
            status = 'RUNNING',
            launch_date = str(datetime.datetime.utcnow()))
        if self.result == 'Passed':
            result = 'PASS'
        else:
            result = 'FAIL'
        if self.commentChanged:
            short_log = self.testAssistant.resultWidget.commentText.text()
        else:
            short_log = ''
        self.testAssistant.testReporter.reportCompleted(
            test_report_id = testReportId,
            status = 'COMPLETED',
            result = result,
            short_log = short_log,
            short_log_format = 'RAW',
            log_zip_file = None,
            completion_date = str(datetime.datetime.utcnow()))

    def submit(self):
        if self.result != 'Not executed':
            self.report()
        sys.exit(0)

class Language:
    def __init__(self,lang):
        if lang is None:
            lang = locale.getdefaultlocale()[0]
            self.lang = lang[0:2]
        else:
            self.lang = lang

    def getLanguage( self):
        #print 'Language: ' + self.lang
        return self.lang

class TestAssistant:
    def __init__(self, xmlFile, testReporter, lang):
        self.test = Test( xmlFile)
        self.language = Language(lang)
        self.testReporter = testReporter
        self.qApplication = qt.QApplication(sys.argv)
        translator = qt.QTranslator(None)
        r = translator.load( "TestAssistant_%s.qm" % self.language.getLanguage(), "%s/edos-testassistant/translations" % defaults['datadir'])
        self.qApplication.installTranslator( translator)
        self.startWidget = Start()
        self.stepWidget = Step()
        self.resultWidget = Result()
        self.startAdapter = StartAdapter( self)
        self.stepAdapter = StepAdapter( self)
        self.resultAdapter = ResultAdapter( self)
        self.nw1Adapter = NextWidgetAdapter( self.qApplication, self.startWidget, self.startWidget.startButton, self.stepWidget)
        self.nw2Adapter = NextWidgetAdapter( self.qApplication, self.stepWidget, self.stepWidget.resultButton, self.resultWidget)
        self.qApplication.setMainWidget( self.startWidget)
        self.qApplication.mainWidget().show()
        self.qApplication.exec_loop()
