#
# EDOS testing framework
# Copyright (C) 2006 EDGE-IT
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

#
# Author: Francois Dechelle <fdechelle@mandriva.com>
#

from xml.dom.ext.reader import PyExpat
from xml import xpath
import xml.dom.ext
import sys

class Test:
    def __init__(self, xml_file):
        reader = PyExpat.Reader()
        self.domDocument = reader.fromStream( open(xml_file))
        testNode = xpath.Evaluate( '/manual-test', self.domDocument)[0]
        self.__buildLanguages__( testNode)
        self.__buildDescription__( testNode)
        self.__buildSteps__( testNode)
        self.__buildExpectedResult__( testNode)

    def __buildTextDict__( self, node):
        d = dict()
        for textNode in xpath.Evaluate( 'text', node):
            lang = 'en'
            if textNode.hasAttribute('lang'):
                lang = textNode.getAttribute('lang')
            d[lang] = reduce(lambda x,y: x+y.nodeValue, textNode.childNodes, '')
        return d

    def __buildDescription__( self, testNode):
        descNode = xpath.Evaluate( './description', testNode)[0]
        self.name = descNode.getAttribute('name')
        self.suite = descNode.getAttribute('suite')
        self.description = self.__buildTextDict__( descNode)

    def __buildLanguages__(self, testNode):
        self.languages = []
        for l in xpath.Evaluate( './languages/language', testNode):
            self.languages.append( l.getAttribute( 'id'))

    def __buildSteps__( self, testNode):
        self.steps = []
        for node in xpath.Evaluate( './steps/step', testNode):
            instruction = xpath.Evaluate( './instruction', node)[0]
            self.steps.append( self.__buildTextDict__( instruction) )

    def __buildExpectedResult__( self, testNode):
        expectedResult = xpath.Evaluate( './result', testNode)
        self.expectedResult = self.__buildTextDict__( expectedResult[0])

    def __str__(self):
        s = u'Test: ' + self.name + u'\n'
        s = s + u'Suite: ' + self.suite + u'\n'

        s = s + u'Description:\n'
        for k,v in self.description.iteritems():
            s = s + u'\'' + k + u'\' -> ' + v + u'\n'
        s = s + u'\n'

        s = s + u'Languages: '
        for l in self.languages:
            s = s + l + u' '
        s = s + u'\n\n'
        
        count = 0
        for step in self.steps:
            s = s + u'Step ' + str(count) + u':\n'
            count = count + 1
            for k,v in step.iteritems():
                s = s + u'\'' + k + u'\' -> ' + v + u'\n'

        s = s + u'Expected result:\n'
        for k,v in self.expectedResult.iteritems():
            s = s + u'\'' + k + u'\' -> ' + v + u'\n'
        return s

if __name__ == '__main__':
    test = Test( sys.argv[1])
    print str(test)

